package org.saager.enumerator;

import java.util.List;
import java.util.Set;

/**
 * Extension of Enumerator that tries to create modifiable Wrapper
 * 
 * As List and Set are Iterable, an Enumerator can be used on them, but update-calls
 * will not work. The implementation here are less efficient but work with update
 * 
 * @author Carsten Saager
 *
 * @param <T>
 */
public class LiveEnumerator<T> extends Enumerator<T> {

	/**
	 * LiveEnumerator on a List.
	 * 
	 * Instead of iterating the basic access will be made via index get(int), 
	 * this might be slightly less efficient than iteration
	 * 
	 * @param iterable
	 */
	public LiveEnumerator(List<T> iterable) {
		this(iterable,Predicate.TAUTOLOGIC);
	}
	/**
	 * LiveEnumerator on a List and filter. 
	 * 
	 * @param iterable
	 * @param filter
	 */
	public LiveEnumerator(List<T> iterable, Predicate<? extends Object> filter) {
		super(new LiveIterable<T>(iterable),filter);
	}
	/**
	 * LiveEnumerator on a Set.
	 * 
	 * This implementation might behave unexpectly and also lack some efficiency.
	 * The limitations of a set cause that so it is worth a look if updates are really needed
	 * 
	 * @see LiveIterable for details
	 * 
	 * @param iterable
	 */
	public LiveEnumerator(Set<T> iterable) {
		this(iterable,Predicate.TAUTOLOGIC);
	}
	/**
	 * LiveEnumerator on a Set and filter
	 * 
	 * @param iterable
	 * @param filter
	 */
	public LiveEnumerator(Set<T> iterable, Predicate<? extends Object> filter) {
		super(new LiveIterable<T>(iterable),filter);
	}

}
