package org.saager.enumerator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Set;


public class LiveIterable<T> implements Iterable<T>{
	private interface ArrayHolder {
		Object get(int i);
		boolean set(int i, Object obj);
	}
	private final ArrayHolder ah;
	public LiveIterable(final Set<T> set) {
		final ArrayList<T> array = new ArrayList<T>(set);
		arrLength = array.size();
		ah = new ArrayHolder(){

			public Object get(int i) {
				return array.get(i);
			}

			@SuppressWarnings("unchecked")
			public boolean set(int i, Object obj) {
				int exists = array.indexOf(obj);
				if (exists==i) {
					return false;
				}
				T old = array.set(i, (T) obj);
				if (exists<0) {
					// new value
					final boolean success = set.remove(old)	& set.add((T) obj) ;
					assert success: "changing value changes set-size!";
					return false;
				} else {
					//already seen
					set.remove(array.remove(exists));
					arrLength--;
					return exists<i;
				}
			}};
		
	}
	
	public LiveIterable(final List<T> array) {
		arrLength = array.size();
		ah = new ArrayHolder(){

			public Object get(int i) {
				return array.get(i);
			}

			@SuppressWarnings("unchecked")
			public boolean set(int i, Object obj) {
				array.set(i, (T) obj);
				return false;
			}};
	}


	int arrLength;
	
	public Iterator<T> iterator() {
		return new LiveIterator<T>(){
			int index;
			public boolean hasNext() {
				return index<arrLength;
			}

			@SuppressWarnings("unchecked")
			public T next() {
				try {
					return (T)ah.get(index++);
				} catch (IndexOutOfBoundsException e) {
					throw new NoSuchElementException();
				}
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}

			public void set(T obj) {
				if (index==0) throw new NoSuchElementException();
				if (ah.set(index-1, obj)) index--;
			}};
	}

}
