package org.saager.enumerator;

public abstract class Predicate<T> {
	static final Predicate<Object> TAUTOLOGIC = new Predicate<Object>() {

		@Override
		protected
		boolean matches(Object obj) {
			return true;
		}
	};
	public final Predicate<T> or(final Predicate clause) {
		return new Predicate<T>() {

			@SuppressWarnings("unchecked")
			@Override
			protected
			boolean matches(T obj) {
				return Predicate.this.matches(obj) || clause.matches(obj);
			}
			
		};
	}
	public final Predicate<T> either(final Predicate clause) {
		return new Predicate<T>() {

			@SuppressWarnings("unchecked")
			@Override
			protected
			boolean matches(T obj) {
				return Predicate.this.matches(obj) ^ clause.matches(obj);
			}
			
		};
	}

	public final Predicate<T> and(final Predicate predicate) {
		return new Predicate<T>() {

			@SuppressWarnings("unchecked")
			@Override
			protected
			boolean matches(T obj) {
				return Predicate.this.matches(obj) && predicate.matches(obj);
			}
			
		};
	}
	public final Predicate<T> not() {
		return new Predicate<T>() {

			@Override
			protected
			boolean matches(T obj) {
				return ! (Predicate.this.matches(obj));
			}
			
		};
	}
	protected abstract boolean matches(T obj);
}
