package org.saager.enumerator;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;

public class EnumeratorTest {

	@Before
	public void setUp() throws Exception {
	}

	@Test
	public void testEnumerator() {
		new Array<Integer>(1,2);
		new Array<Long>(1l,2l);
		new Array<Byte>((byte)1);
		new Array<Character>('a');
		new Array<Integer>(new int[]{1,3,4});
		new Array<Boolean>(new long[]{1l,2l});
		new Array<Byte>(Byte.valueOf((byte) 1));
		new Array<Character>(new Character[0]);
		
		List<String> li = new ArrayList<String>();
		li.add("DeleteMe");
		for (Cursor<String> a : new Enumerator<String>(li)) {
			assertEquals(0, a.index);
			assertTrue(a.remove());
		}
		assertTrue(li.isEmpty());
	}

	@Test
	public void testProtect() {
		List<Integer> lii =  new ArrayList<Integer>();
		lii.add(1);lii.add(2);lii.add(3);
		Iterator<Integer> i1 = lii.iterator();
		int sum=0;
		while (i1.hasNext()) {
			if (i1.next()==2) i1.remove();
			for (int ii:lii) {
				sum+=ii;
			}
		}
		assertEquals(6+4+4, sum);
		lii.add(1, 2);
		sum=0;
		for (Cursor<Integer> c1: new Enumerator<Integer>(lii)){
			if (c1.value==2) assertTrue(c1.remove());
			for (Cursor<Integer> c2: new Enumerator<Integer>(lii)) sum+=c2.value;
			
		}
		assertEquals(6+4+4, sum);
		sum=0;
		Array<Double> lid = new Array<Double>(1.0,2.0,3.0);
		for (Cursor<Double> dd: new Enumerator<Double>(lid)) {
			dd.update(dd.value*2);
			for (Cursor<Double> d2:new Enumerator<Double>(lid)) {
				d2.update(d2.value-1);
				sum+=d2.value;
			}
		}
		assertEquals(7+5+3, sum);
		
		List<String> li = new ArrayList<String>();
		li.add("DeleteMe");
		li.add("DeleteThis");
		Cursor<String> first = null;
		for (Cursor<String> a : new Enumerator<String>(li)) {
			if (a.isFirst()) {
				first = a;
				assertTrue(a.hasSuccessor());
			} else {
				assertFalse(a.hasSuccessor());
				boolean catched = false;
				try {
					first.remove();
				} catch (IllegalStateException ise) {
					catched = true;
				}
				assertTrue(catched);
			}
		}
		boolean catched = false;
		try {
			first.remove();
		} catch (IllegalStateException ise) {
			catched = true;
		}
		assertTrue(catched);
	}

	@Test
	public void testIterator() {
		String[] li = { "DeleteMe" };
		for (Cursor<String>a : new Enumerator<String>(new Array<String>(li))) {
			assertEquals(0, a.index);
			assertFalse(a.remove());
			assertTrue(a.isFirst());
			assertFalse(a.hasSuccessor());
		}
		assertEquals(1, li.length);
	}

	@Test
	public void testArrays() {
		
		int[] li = { 1 };
		for (int a:li){
			a=0;
		}
		for (Cursor<Integer> a : new Enumerator<Integer>(new Array<Integer>(li))) {
			assertEquals(0, a.index);
			assertFalse(a.remove());
			assertTrue(a.isFirst());
			assertFalse(a.hasSuccessor());
		}
		assertEquals(1, li.length);
		
	}	
	@Test
	public void testParallel() {
		
		int[] li = { 1,2,3 };
		ArrayList<String> al = new ArrayList<String>();
		al.add("1");
		al.add("2");
		al.add("3");
		for (Pair<String,Integer> a : new Parallel<String, Integer>(new Enumerator<String>(al),new Enumerator<Integer>(new Array<Integer>(li)))) {
			assertEquals(a.isFirst(), a.value.index==0);
			if (a.secondValue.value==2){
				assertTrue(a.remove());
			} else {
				assertEquals(a.hasSuccessor(), a.value.index<2);
			}
			assertEquals(a.value.value, String.valueOf(a.secondValue.value));
		}
		assertEquals(2, al.size());
		
	}	
	@Test
	public void testFilter() {
		ArrayList<Number> al = new ArrayList<Number>();
		al.add(new Double(1.2));
		al.add(new Integer(2));
		al.add(new Double(2.1));
		
		for (Cursor<Double> d : new Enumerator<Number>(al).specialize(Double.class)) {
			switch(d.index) {
			case 0: assertEquals(new Double(1.2),d.value);
				assertTrue(d.remove()); 
				break;
			case 1: assertEquals(new Double(2.1),d.value); break;
			default: fail();
			}
		}
		assertEquals(2, al.size());
	}
	@SuppressWarnings("unchecked")
	@Test
	public void testConcat() {
		ArrayList<Number> a1 = new ArrayList<Number>();
		a1.add(new Double(1.2));
		a1.add(new Integer(2));
		a1.add(new Double(2.1));
		ArrayList<Number> a2 = new ArrayList<Number>();
		a2.add(new Long(3));
		a2.add(new Long(4));
		ArrayList<Long> a3 = new ArrayList<Long>();
		a3.add(new Long(5));
		for (Cursor<Number> nn: new Enumerator<Number>(a1).append(new Enumerator<Number>(a2),new Enumerator<Number>(a3))) {
			switch (nn.index) {
			case 3:
			case 4:
			case 5: assertEquals(nn.index,nn.value.intValue());break;
			}
		}
		
		for (Cursor<Number> nc : new LiveEnumerator<Number>(a1).append(new LiveEnumerator<Long>(a3))) {
			nc.update(nc.value.intValue()+1);
		}
		
	}
	
	@Test
	public void testPredicates() {
		ArrayList<Integer> al = new ArrayList<Integer>();
		for (int ii=2;ii<20;al.add(ii++));
		Predicate<Integer> div2 = new Predicate<Integer>() {

			@Override
			protected boolean matches(Integer obj) {
				return obj%2==0;
			}	
		};
		Predicate<Integer> div3 = new Predicate<Integer>() {

			@Override
			protected boolean matches(Integer obj) {
				return obj%3==0;
			}	
		};
		
		Predicate<Integer> div5 = new Predicate<Integer>() {

			@Override
			protected boolean matches(Integer obj) {
				return obj%5==0;
			}	
		};
		
		for (Cursor<Integer> ii: new Enumerator<Integer>(al).only(div2).only(div3).only(div5)) {
			fail(String.valueOf(ii.value));
		}
		int cnt=0;
		for (Cursor<Integer> ii: new Enumerator<Integer>(al).only(div5).only(div2).also(div5.and(div3.not()))) {
			switch (ii.value) {
			case 10:
			case 5:
				cnt++;break;
			default:fail(String.valueOf(ii.value));
			}
		}
		assertEquals(2, cnt);
		for (Cursor<Integer> ii: new Enumerator<Integer>(al,div5).either(div2).either(div3)) {
			switch (ii.value) {
			case 2:
			case 4:
			case 8:
			case 16:
			case 3:
			case 9:
			case 5:
			case 14:
				cnt++;break;
			default:fail(String.valueOf(ii.value));					
			}
		}
		assertEquals(2+8, cnt);
	}
	@Test
	public void testSets() {
		Set<Integer> set  = new HashSet<Integer>();
		set.add(1);
		set.add(2);
		set.add(3);
		set.add(5);
		for (Cursor<Integer> ci: new LiveEnumerator<Integer>(set)) {
			if (ci.value==2) ci.update(4);
			if (ci.value==3) ci.remove();
		}
		int sum=0;
		for (Cursor<Integer> ci: new LiveEnumerator<Integer>(set)) sum+=ci.value;
		assertEquals(13, sum);
		
	}
}
